@@ -8,7 +8,7 @@
 #        .
 #          .
 #
-#	SCCS: @(#)Expression.pm	1.37 03/27/15 15:46:37
+#	SCCS: @(#)Expression.pm	1.38 03/27/15 18:50:40
 #
 # This module is free software; you can redistribute it and/or modify
 # it under the same terms as Perl itself. You must preserve this entire copyright
@@ -50,7 +50,7 @@ use vars qw/
 	$Version
 );
 
-our $VERSION = "1.37";
+our $VERSION = "1.38";
 
 # Fundamental to this is a tree of nodes.
 # Nodes are hashes with members:
@@ -70,9 +70,9 @@ our $VERSION = "1.37";
 # Getting the precedence values right is a pain and for things like close paren, non obvious.
 
 my %OperPrec = (
-	'var'	=>	[23, 23],
-	'const'	=>	[23, 23],
-	'['	=>	[7, 22],
+	'var'	=>	[24, 24],
+	'const'	=>	[24, 24],
+	'['	=>	[7, 23],
 	'++'	=>	[22, 22],
 	'--'	=>	[22, 22],
 	'M-'	=>	[20, 21],	# Monadic -
@@ -109,10 +109,10 @@ my %OperPrec = (
 	'func'	=>	[21, 22],
 	']'	=>	[7, 7],
 	':='	=>	[5, 6],		# 6 to make := right assosc
-	';'	=>	[2, 2],
-	'{'	=>	[1, 0],
 	'}'	=>	[4, 0],
 	'flow'	=>	[3, 4],
+	';'	=>	[2, 2],
+	'{'	=>	[1, 0],
 	'EOF'	=>	[-50, -50],
 );
 # TOS, JR
@@ -465,8 +465,8 @@ sub CheckTreeInt {
 		$ok = 0;
 	}
 
-	if(defined($MonVarOp{$tree->{oper}}) and (!defined($tree->{right}) or $tree->{right}{oper} ne 'var')) {
-		$self->{PrintErrFunc}("Operand to '%s' must be a variable", $tree->{oper});
+	if(defined($MonVarOp{$tree->{oper}}) and (!defined($tree->{right}) or ($tree->{right}{oper} ne '[' and $tree->{right}{oper} ne 'var'))) {
+		$self->{PrintErrFunc}("Operand to '%s' must be a variable or indexed array element", $tree->{oper});
 		$ok = 0;
 	}
 
@@ -592,6 +592,32 @@ sub EvalTree {
 	return $self->{FuncEval}($self, $tree->{fname},
 				$self->EvalTree($tree->{right}, $tree->{fname} eq 'defined'))		if($oper eq 'func');
 
+	if($oper eq '++' or $oper eq '--') {
+		my ($right, @right, @left, $index, $name);
+		# The variable is either a simple variable or an indexed array
+		if($tree->{right}->{oper} eq '[') {
+			$name = $tree->{right}->{left}->{name};
+			$index = $self->EvalTree($tree->{right}->{right}, 0);
+			unless($index =~ /^\d+$/) {
+				$self->{PrintErrFunc}("Array '%s' index is not integer '%s'", $name, $index);
+				return undef;
+			}
+			@left = $self->EvalTree($tree->{right}->{left}, 0);
+			$right = $left[$index];
+			$name = "$name\[$index\]";
+		} else {
+			@right = $self->EvalTree($tree->{right}, 0);
+			$right = $right[-1];
+			$name = $tree->{right}{name};
+		}
+
+		$oper eq '++' ? $right++ : $right--;
+
+		$self->{VarSetFun}($self, $name, ($right));
+
+		return $right;
+	}
+
 	# Monadic operators:
 	if(!defined($tree->{left}) and defined($tree->{monop})) {
 		$oper = $tree->{monop};
@@ -617,11 +643,6 @@ sub EvalTree {
 		return  $right if($oper eq '+');
 		return !$right if($oper eq '!');
 		return ~$right if($oper eq '~');
-		if($oper eq '++' or $oper eq '--') {
-			$oper eq '++' ? $right++ : $right--;
-
-			return $self->{VarSetFun}($self, $tree->{right}{name}, $right);
-		}
 
 		$self->{PrintErrFunc}("Unknown monadic operator when evaluating: '%s'", $oper);
 		return;
@@ -1260,11 +1281,12 @@ matching singe or double quotes. Escapes are not looked for in literal strings.
 
 The operators should not surprise any Perl/C programmer, with the exception that assignemnt
 is C<:=>. Operators associate left to right except for C<:=> which associates right to left.
-Precedence may be overridden with parenthesis.
+Precedence may be overridden with parenthesis C<( )>@.
 <> is the same as C<!=>.
 
 	++ --	Pre increment/decrement only
 	+ - ~ !	(Monadic)
+	**
 	* / %
 	+ -
 	.	String concatenation
@@ -1332,7 +1354,7 @@ Arrays cannot contain other arrays.
 Array indexes start with C<0>.
 
 	a := (20,21,22); a[1] + a[2]
-	a := (20,21,22); a[1] := 9
+	a := (20,21,22); a[1] := 9; ++a[k + j]
 	i := -1; j := 2; a := (20,21,22); a[i + j] := 3
 
 When setting values you can extend an array one element at a time.
@@ -1424,7 +1446,7 @@ Alain D D Williams <addw@phcomp.co.uk>
 
 =head2 Copyright and Version
 
-Version "1.37", this is available as: $Math::Expression::Version.
+Version "1.38", this is available as: $Math::Expression::Version.
 
 Copyright (c) 2003, 2015 Parliament Hill Computers Ltd/Alain D D Williams. All rights reserved.
 This program is free software; you can redistribute it and/or modify it
